#!/usr/bin/perl
use strict;
use vars qw($opt_v $opt_h $opt_e $opt_s $opt_d $opt_u);
use Getopt::Std;
use Lingua::LanguageGuesser;
getopts('hved:u:s:');

sub help {
    print <<HELP
Usage
-----

* print help message:

$0 -h

* for guessing: 

$0 [-d Dir] [-u Dir] [-s 'lang1, lang2, ,,,'] [-e] [-v]

    -d    indicates in which directory the your additional language models
          are located.
    -u    indicates in which directory the your additional utf8 encoding 
          language models are located.
    -s    set suspected languages list.
    -e    determines how much worse result must be in order not to be 
          mentioned as an alternative. 
    -v    verbose. list all candicates with score.
HELP
}

if ($opt_h) { help(); exit 0; };


my $textstring = "";
my %param = ();
{
    local $/ = undef;
    $textstring = <>;
}

$param{'MyModel'}      = $opt_d;
$param{'MyModel_utf8'} = $opt_u;

my $guesser = Lingua::LanguageGuesser->guess(\%param, $textstring);

if ($opt_s) {
    my @langs;
    foreach my $lang (split ",", $opt_s) {
        $lang =~ s/\s//g;
        push @langs, $lang;
    }
    $guesser->suspect(@langs);
}
$guesser->eliminate() if $opt_e;

if ($opt_v) {
    my %score = $guesser->score_of_lang();
    foreach my $lang ( $guesser->result_list() ) {
         printf "%-15.15s\t%20.20s\n", $lang, $score{$lang};
    }
}
else {
    print $guesser->best_scoring(), "\n";
}

