# Before `make install' is performed this script should be runnable with
# `make test'. After `make install' it should work as `perl all.t'
use blib;
use Test::More tests => 108;

# 1st test
BEGIN { use_ok('Lingua::LanguageGuesser') };
use Lingua::LanguageGuesser qw(create_model delete_model list_my_models);

my $language;
my $input;
my $text;
my $guesser;
my $most_similar_lang;
my @result_languages;
my %score_of_lang;

=pod

=head1 Test about all languages supported by default.

=cut

my @languages_supported = qw(
    afrikaans
    albanian
    arabic-iso8859_6
    amharic-utf
    arabic-windows1256
    armenian
    basque
    belarus-windows1251
    bosnian
    breton
    bulgarian-iso8859_5
    catalan
    chinese-big5
    chinese-gb2312
    chinese_simple-utf8
    chinese_ZH-utf8
    croatian-ascii
    czech-iso8859_2
    danish
    dutch
    english
    esperanto
    estonian
    finnish
    finnish-utf8
    french
    french-utf8
    frisian
    georgian
    german
    german-utf8
    greek-iso8859-7
    hebrew-iso8859_8
    hindi
    hungarian
    icelandic
    indonesian
    irish
    italian
    italian-utf8
    japanese-euc_jp
    japanese-shift_jis
    japanese-utf8
    korean
    korean-utf8
    latin
    latvian
    lithuanian
    malay
    manx
    marathi
    middle_frisian
    mingo
    nepali
    norwegian
    norwegian-utf8
    persian
    polish
    portuguese
    quechua
    romanian
    rumantsch
    russian-iso8859_5
    russian-koi8_r
    russian-windows1251
    russian-utf8
    sanskrit
    scots
    scots_gaelic
    serbian-ascii
    slovak-ascii
    slovak-windows1250
    slovenian-ascii
    slovenian-iso8859_2
    spanish
    spanish-utf8
    swahili
    swedish
    swedish-utf8
    tagalog
    tamil
    thai
    turkish
    ukrainian-koi8_u
    vietnamese
    welsh
    yiddish-utf
);

foreach $language (@languages_supported) {
    my $text = get_test_data($language);
    $most_similar_lang = Lingua::LanguageGuesser
                        ->guess($text)
                        ->best_scoring(); 
    is ($most_similar_lang , $language, "$language test");
}

$text = 'hoge';
$guesser = Lingua::LanguageGuesser->guess({utf8 => 'omit'});
$guesser->set_text($text);
$most_similar_lang = $guesser->best_scoring();
is ($most_similar_lang , 'hungarian', "test");

$text = 'hoge';
$guesser->set_text($text);
$most_similar_lang = $guesser->best_scoring();
is ($most_similar_lang , 'hungarian', "test");

$text = get_test_data('korean');
$guesser->set_text($text);
%score_of_lang = $guesser->score_of_lang();
cmp_ok ($score_of_lang{'korean'}, '>=', 100000, 'score_of_lang() korean test1'); 
cmp_ok ($score_of_lang{'korean'}, '<=', 120000, 'score_of_lang() korean test2'); 
cmp_ok ($score_of_lang{'basque'}, '>=', 150000, 'score_of_lang() basque test1'); 
cmp_ok ($score_of_lang{'basque'}, '<=', 160000, 'score_of_lang() basque test2'); 

# eliminate test 1
$text = 'hoge';
@result_languages
    = Lingua::LanguageGuesser
        ->guess({utf8 => 'omit'}, $text)
        ->eliminate()
        ->result_list();
my $eliminate1 = scalar @result_languages;

# eliminate test 2
@result_languages =
    Lingua::LanguageGuesser
        ->guess({utf8 => 'omit'}, $text)
        ->eliminate(1.1)
        ->result_list();
my $eliminate2 = scalar @result_languages;
cmp_ok ($eliminate2, '>=', $eliminate1, "eliminate() test1");
cmp_ok ($eliminate2, '<=', 74, "eliminate() test2");

# eliminate test 3
$text = get_test_data('korean');
$most_similar_lang =
    Lingua::LanguageGuesser
        ->guess({utf8 => 'omit'}, $text)
        ->eliminate(3.0)
        ->best_scoring();
is ($most_similar_lang, "Two or more suspects remain", "eliminate test3");

# eliminate test 4
$text = get_test_data('korean');
$guesser = Lingua::LanguageGuesser->guess({utf8 => 'omit'}, $text);
$guesser->eliminate(3.0);
$guesser->set_text($text);
$most_similar_lang = $guesser->best_scoring();
is ($most_similar_lang, "korean", "eliminate test4");

# suspect test
$text = get_test_data('korean');
@result_languages =
    Lingua::LanguageGuesser
        ->guess({utf8 => 'omit'}, $text)
        ->suspect('japanese-euc_jp', 'english')
        ->result_list();
cmp_ok (scalar @result_languages, '==', 2, "suspect() test");

# Combination test 1
$text = get_test_data('korean');
@result_languages
    = Lingua::LanguageGuesser
        ->guess({utf8 => 'omit'}, $text)
        ->suspect('japanese-euc_jp', 'english')
        ->eliminate()
        ->result_list();
cmp_ok (scalar @result_languages, '==', 1, "Combination test1");

# Combination test 2
$text = get_test_data('korean');
$guesser = Lingua::LanguageGuesser->guess({utf8 => 'omit'}, $text);
$guesser->eliminate()
        ->suspect('japanese-euc_jp', 'english');
@result_languages = $guesser->result_list();
cmp_ok (scalar @result_languages, '==', 0, 'Combination test2');

# Combination test 3
$most_similar_lang = $guesser->best_scoring();
is ($most_similar_lang, 'No language suspects', 'Combination test3');

# MaxLine test
$text = get_test_data('spanish');
@result_languages =
    Lingua::LanguageGuesser
        ->guess({ MaxLine => '1', utf8 => 'omit'}, $text )
        ->eliminate()
        ->result_list();
cmp_ok (scalar @result_languages, '>=', 5, 'MaxLine test 1');
cmp_ok (scalar @result_languages, '<=', 7, 'MaxLine test 2');

# language model test

# Create or remake language model.
create_model('t/testLM', 't/testLM/japanese-euc_jp.txt', 'japanese-euc_jp' );

$text = "hoge";
$result = Lingua::LanguageGuesser->guess({ MyModel => 't/testLM', utf8 => 'omit' }, $text)->best_scoring();
is ($result, 'japanese-euc_jp', 'MyModel test');

$result = Lingua::LanguageGuesser->guess({ MyModel_utf8 => 't/testLM', utf8 => 'include'}, $text)->best_scoring();
is ($result, 'japanese-euc_jp', 'MyModel_utf8 test');

# usable language list test
@my_models = list_my_models('t/testLM');
is (shift @my_models, 'japanese-euc_jp', 'list_my_models test'); 

# Delete language model.
delete_model('t/testLM', 'japanese-euc_jp');

$text = "hoge";
$result = Lingua::LanguageGuesser->guess({ MyModel => 't/testLM', utf8 => 'omit' }, $text)->best_scoring();
is ($result, 'hungarian', 'MyModel test');

# Get test data from "./ShortText" (only top 10 lines)
sub get_test_data {
    my $language = shift;
    my $text = "";
    open (IN, "./ShortTexts/${language}.txt") or die $!;
    foreach (0..9) {
         $text .= <IN>
    }
    return $text;
}

